
Public Class clsPlatform

    Declare Function GetKeyboardLayoutList Lib "user32" (ByVal nBuff As Integer, ByRef lpList As Integer) As Integer
    Declare Function ActivateKeyboardLayout Lib "user32" (ByVal HKL As Integer, ByVal flags As Integer) As Integer
    Declare Function GetLocaleInfo Lib "kernel32" Alias "GetLocaleInfoA" (ByVal Locale As Integer, ByVal LCType As Integer, ByVal lpLCData As String, ByVal cchData As Integer) As Integer
    Const LOCALE_SENGCOUNTRY As Long = &H1002 '// English name of country
    Const LOCALE_SLANGUAGE As Long = &H2  'localized name of language
    Const LOCALE_SABBREVLANGNAME As Long = &H3  'abbreviated language name

    Declare Function GetKeyboardLayout Lib "user32" (ByVal dwLayout As Integer) As Integer
    Declare Function GetKeyboardLayoutName Lib "user32" Alias "GetKeyboardLayoutNameA" (ByVal pwszKLID As String) As Long

    Public ErrorMessage As String

    '-- Store these so can use when switching in editors
    Public OriginalKeyboardCode As Long = 0
    Public SyriacKeyboardCode As Long = 0

    '-- See if required fonts present on system -- return collection of missing items
    Public Function FontCheck() As ArrayList
        Dim Fonts As New System.Drawing.Text.InstalledFontCollection()
        Dim Family As FontFamily
        Dim s As String = ""
        Dim serto As Boolean = False
        Dim titus As Boolean = False

        ErrorMessage = ""

        For Each Family In Fonts.Families
            If Family.Name = "Serto Jerusalem" Then
                serto = True
            End If
            If Family.Name = "TITUS Cyberbit Basic" Then
                titus = True
            End If
        Next
        Dim retval As New ArrayList
        If serto = False Then retval.Add("Serto Jerusalem")
        If titus = False Then retval.Add("TITUS Cyberbit Basic")

        If retval.Count <> 0 Then
            ErrorMessage = "The following fonts are required by QuickSyriac but not found on your system:" + vbCrLf
            For Each x As String In retval
                ErrorMessage = ErrorMessage + x + vbCrLf
            Next
        End If

        Return retval
    End Function

    Public Function KeyboardCheck() As ArrayList
        '-- Make sure Syriac installed.  We do not care which Syriac keyboard the user uses
        '-- http://www.experts-exchange.com/Programming/Programming_Languages/Visual_Basic/VB_Controls/Q_21208243.html?query=Regional+Options&topics=94
        '-- http://www.experts-exchange.com/Programming/Programming_Languages/Visual_Basic/VB_Controls/Q_21409380.html
        '-- http://custom.programming-in.net/articles/art9-2.asp?lib=user32.dll  (reference for vb.net calls)
        '-- http://vbnet.mvps.org/index.html?code/locale/localecountry.htm (stuff on country info)

        Dim rc As Long
        Dim i As Integer

        Dim lLayouts(50) As Integer
        Dim retval As New ArrayList
        Dim buf As String = "                                 "
        Dim layout As String
        Dim SyriacFound As Boolean = False

        ErrorMessage = ""

        'Save current configuration
        OriginalKeyboardCode = GetKeyboardLayout(0)
        rc = GetKeyboardLayoutName(buf)
        layout = buf.Substring(0, 8)
        Dim layoutCode As String = "&H" & layout  '-- a long
        Dim layoutName As String = pfGLI(layoutCode, LOCALE_SLANGUAGE)
        If layoutName.Contains("English") = False Then
            MsgBox("Your keyboard is currently not set to English, but to " + layoutName _
            + ".  It will be reset to English.", MsgBoxStyle.Exclamation)
        End If

        'Get the first 50 supported keyboard layouts (50 is max supported for now)
        rc = GetKeyboardLayoutList(50, lLayouts(0))
        'Loop through all the keyboard layouts
        'Ignore the first one on 0 which is negative
        For i = 1 To UBound(lLayouts)
            If lLayouts(i) = 0 Then   '-- all entries beyond those installed are 0
                Exit For
            End If
            '--Activate the keyboard layout and get its name
            rc = ActivateKeyboardLayout(lLayouts(i), 0)
            rc = GetKeyboardLayoutName(buf)
            layout = buf.Substring(0, 8)
            layoutCode = "&H" & layout  '-- a long
            layoutName = pfGLI(layoutCode, LOCALE_SLANGUAGE)
            If layoutName.Contains("Syriac") Then
                SyriacKeyboardCode = lLayouts(i)
                SyriacFound = True
            End If
            If layoutName.Contains("English") And OriginalKeyboardCode = 0 Then
                OriginalKeyboardCode = lLayouts(i)
            End If
            retval.Add(layoutCode + ":" + layoutName)
        Next i

        'Restore current configuration
        ActivateOriginalKeyboard()

        If SyriacFound = False Then
            ErrorMessage = "The Syriac language and phonetic keyboard are not installed on your PC.  Please correct this."
        End If

        Return retval
    End Function

    Private Function pfGLI(ByVal m_LocaleLCID As Long, ByVal reqInfo As Integer) As String
        Dim Buffer As String = "                                                              "
        GetLocaleInfo(m_LocaleLCID, reqInfo, Buffer, Buffer.Length)
        pfGLI = StripNull(Buffer)
    End Function

    Public Function StripNull(ByVal StrIn As String) As String
        Dim nul As Long
        nul = InStr(StrIn, vbNullChar)
        Select Case nul
            Case Is > 1
                StripNull = Left$(StrIn, nul - 1)
            Case 1
                StripNull = ""
            Case 0
                StripNull = Trim$(StrIn)
            Case Else
                StripNull = StrIn
        End Select

    End Function

    Public Function ActivateOriginalKeyboard() As Long
        If OriginalKeyboardCode = 0 Then Return -1
        Return (ActivateKeyboardLayout(OriginalKeyboardCode, 0))
    End Function

    Public Function ActivateSyriacKeyboard() As Long
        If SyriacKeyboardCode = 0 Then Return -1
        Return (ActivateKeyboardLayout(SyriacKeyboardCode, 0))
    End Function

End Class
