Imports System.IO

Public Class clsUniquesMap

    Public uniquesFileName As String = Application.StartupPath + "\uniques.txt"
    Public uniques As New ArrayList()
    Public map As New Dictionary(Of String, ArrayList)

    Public Sub add(ByVal u As clsUniques)

        '-- Add to the array list of all uniques
        uniques.Add(u)
        addToMap(u.stem, u)
        addToMap(u.stem_small, u)

    End Sub

    Private Sub addToMap(ByVal key As String, ByVal u As clsUniques)

        Dim temparray As New ArrayList

        '-- bomb-proofing
        If key Is Nothing Then
            Exit Sub
        End If

        '-- Does it exist in the map under the full stem?  
        If map.ContainsKey(key) = False Then
            '-- No: create an entry
            temparray.Add(u)
            map.Add(key, temparray)
        Else
            '-- Yes: add to arraylist in entry
            temparray = map.Item(key)
            temparray.Add(u)
            map.Item(key) = temparray
        End If

    End Sub

    Public Sub delete(ByVal i As Long)

        Dim u As clsUniques = uniques(i)

        '-- remove from arraylist
        uniques.Remove(u)

        '-- remove from maps
        removeFromMap(u.stem, u)
        removeFromMap(u.stem_small, u)

    End Sub

    Public Sub delete(ByVal u As clsUniques)
        '-- Must pass in *old* stem and stem_small

        '-- remove from arraylist
        uniques.Remove(u)

        '-- remove from maps
        removeFromMap(u.stem, u)
        removeFromMap(u.stem_small, u)

    End Sub

    Private Sub removeFromMap(ByVal key As String, ByVal u As clsUniques)

        Dim temparray As New ArrayList

        '-- bomb-proofing
        If key Is Nothing Then
            Exit Sub
        End If
        If map.ContainsKey(key) = False Then
            Exit Sub
        End If

        temparray = map.Item(key)
        '-- if only item for key get rid of from map
        If temparray.Count <= 1 Then
            map.Remove(key)
            Exit Sub
        End If
        '-- otherwise shorten array and restore
        temparray.Remove(u)
        map.Item(key) = temparray

    End Sub

    Public Sub update(ByVal oldentry As clsUniques, ByVal newentry As clsUniques)
        delete(oldentry)
        add(newentry)
    End Sub

    Public Overrides Function ToString() As String
        Dim s As String = ""
        For Each x As clsUniques In uniques
            s += x.ToString() + vbCrLf
        Next
        ToString = s
    End Function

    Public Sub SetValues(ByVal s As String)
        Dim myarray As String() = Split(s, vbCrLf)
        For Each x As String In myarray
            If x <> "" Then  '-- skip trailing blank line caused by last vbcrlf
                Dim u As New clsUniques
                u.setValues(x)
                add(u)
            End If
        Next
    End Sub

    Public Sub ReadFromDisk()

        Try
            Dim s As String
            'Dim s As String = My.Computer.FileSystem.ReadAllText(uniquesFileName)
            Dim sr As New StreamReader(uniquesFileName, System.Text.Encoding.Unicode)
            s = sr.ReadToEnd
            sr.Close()
            SetValues(s)
        Catch ex As System.IO.FileNotFoundException
            MsgBox("Unable to locate file '" + uniquesFileName + "'", MsgBoxStyle.Exclamation)
        Catch ex As Exception
            MsgBox(ex.Message, MsgBoxStyle.Exclamation)
        End Try

    End Sub

    Public Sub SaveToDisk()
        'My.Computer.FileSystem.WriteAllText(uniquesFileName, ToString, False)
        ' Write the string as Unicode (2 bytes per character)
        ' This also writes the Unicode preamble at the beginning of the file.
        Dim appendMode As Boolean = False ' This overwrites the entire file.
        Dim sw As New StreamWriter(uniquesFileName, appendMode, System.Text.Encoding.Unicode)
        sw.Write(ToString)
        sw.Close()
    End Sub

End Class
